/******************************************************************************/
/*
                                  content.h
*/
/******************************************************************************/

#ifndef INCLUDE_CONTENT_H
#define INCLUDE_CONTENT_H

#define CONTENT_ORDERED_FIRST_TO_LAST        1
#define CONTENT_ORDERED_LAST_TO_FIRST        2
#define CONTENT_ORDERED_BLOG_MOST_RECENT     3
#define CONTENT_ORDERED_BLOG_LEAST_RECENT    4
#define CONTENT_ORDERED_UPDATED_MOST_RECENT  5
#define CONTENT_ORDERED_UPDATED_LEAST_RECENT 6

#define SECTION_MAX 1+6

/*******************/
/* data structures */
/*******************/

#define TAG_STRING_SIZE 63

typedef struct STRUCT_TAG_LIST_DATA
{
    struct STRUCT_TAG_LIST_DATA  *NextPtr,
                                 *PrevPtr;
    int  TagCount,
         TagLength;
    char  TagString [1];
} TAG_LIST_DATA;

typedef struct STRUCT_TAG_DATA
{
    BOOL  TagInit;
    int  ListCount,
         MemBytes,
         TagCount,
         TagMaxCount;
    struct STRUCT_TAG_LIST_DATA  *ListPtr;
} TAG_DATA;

typedef struct STRUCT_CONTENT_SYNOPSIS
{
   struct STRUCT_CONTENT_SYNOPSIS  *NextPtr,
                                   *PrevPtr;

   struct STRUCT_CONTENT_DATA  *ContentPtr;

   BOOL IsBlogEntry;

   int  SectionNumber [SECTION_MAX];

   unsigned long  SectionLastUpdated,
                  TimeStampSeconds;

   char  *ExtractPtr;

} CONTENT_SYNOPSIS;

typedef struct STRUCT_CONTENT_DATA
{
   struct STRUCT_CONTENT_DATA  *NextPtr,
                               *PrevPtr,
                               *CachePtr;

   struct STRUCT_CONTENT_SYNOPSIS  /* only used with the first section */
                                   *SynopsisPtr;

   BOOL  /* has an $$IF$$ on the first line */
         HasLeadingIf,
         /* it's a $$CLIP$ */
         IsClip,
         /* not generally visible (during non-management) */
         IsHidden,
         /* section is not to be printed */
         NoPrint,
         /* section and subordinates are not to be printed */
         NoPrintPlus,
         /* no entry in the table-of-content but otherwise visible */
         NoToc;
         
   int  /* from HTML content or mtime */
        ContentLastUpdated,
        /* <h1> == 1, <h2> == 2 ... <h6> == 6 */
        HeadNumber,
        /* if CGIplus and persistent */
        HitCount,
        /* if CGIplus and persistent */
        mtime,
        /* if CGIplus and persistent */
        ReusedCount,
        /* from section heading title attribute */
        SectionLastUpdated,
        /* length of composite section identifier */
        SectionNameLength,
        /* length of the section title */
        SectionTitleLength,
        /* the ordering of the synopsis list */
        SynopsisOrderedAs,
        /* EndTextPtr minus TextPtr */
        TextLength;

   int  /* total headings, <h1> number, <h2> number ... <h5> number */
        SectionNumber [SECTION_MAX];

   char  /* points to any $$BLOG$$ on first line */
         *BlogPtr,
         /* pointer to the end of the text of the section */
         *EndTextPtr,
         /* points to any $$GONETO$$ on first line */
         *GoneToPtr,
         /* points to any $$GOTO$$ on first line */
         *GoToPtr,
         /* pointer to section RSS directive on first line */
         *RssPtr,
         /* composite identifier from heading tags */
         *SectionName,
         /* string from between heading tags */
         *SectionTitle,
         /* pointer to the text of the section */
         *TextPtr,
         /* points to any $$TITLE$$ on first line */
         *TitlePtr;

   char  SectionNumberDot [32];

   TAG_DATA  /* stores tags generated directly from content */
             ContentTagData,
             /* stores $$TAG$$ tags */
             SiteTagData;

} CONTENT_DATA;

typedef struct STRUCT_CONTENT_CACHE
{
   struct STRUCT_CONTENT_DATA  *CachePtr;

   int  mtime,
        HitCount,
        LastUpdated,
        ReusedCount,
        SiteDepth,
        TextLength;

   char  *TextPtr;

   char  /* file content was loaded from */
         FileName [256];

} CONTENT_CACHE;

/**************/
/* prototypes */
/**************/

void ContentCacheInit ();
void ContentCrumbs (char*);
int ContentDeEntify (char*, BOOL);
void ContentFileDotType ();
CONTENT_DATA* ContentFindSection (char*);
CONTENT_DATA* ContentDuplicate (CONTENT_DATA*);
BOOL ContentLatestBlog (char*);
BOOL ContentLoad ();
void ContentTocCss (char*);
void ContentTocDrop (char*);
void ContentTocInline (char*);
void ContentTocPrint ();
BOOL ContentOnlyIfLoggedIn ();
void ContentParse (char*);
void ContentParseLine1 (CONTENT_DATA*);
void ContentProcess ();
void ContentRssFeed (char*);
BOOL ContentSanity (CONTENT_DATA*);
void ContentSiteMap ();
void ContentSiteRedirect ();
void ContentTag (char*);
void ContentUnParse (CONTENT_DATA*);

#endif /* INCLUDE_CONTENT_H */

/******************************************************************************/

